package sync

import (
	"fmt"
	"path"
	"strings"
	"strconv"

	"srs.epita.fr/fic-server/libfic"
)

func getExercices(i Importer, theme fic.Theme) ([]string, error) {
	var exercices []string

	if dirs, err := i.listDir(theme.Name); err != nil {
		return []string{}, err
	} else {
		for _, dir := range dirs {
			if _, err := i.listDir(path.Join(theme.Name, dir)); err == nil {
				exercices = append(exercices, dir)
			}
		}
	}

	return exercices, nil
}

func SyncExercices(i Importer, theme fic.Theme) []string {
	var errs []string

	if exercices, err := getExercices(i, theme); err != nil {
		errs = append(errs, err.Error())
	} else {
		dmap := map[int]fic.Exercice{}
		emap := map[string]int{}
		for _, edir := range exercices {
			edir_splt := strings.SplitN(edir, "-", 2)
			if len(edir_splt) != 2 {
				errs = append(errs, fmt.Sprintf("%q is not a valid exercice directory: missing id prefix", edir))
				continue
			}

			eid, err := strconv.Atoi(edir_splt[0])
			if err != nil {
				errs = append(errs, fmt.Sprintf("%q: invalid exercice identifier: %s", edir, err))
				continue
			}
			ename := edir_splt[1]

			emap[ename] = eid

			// Overview and scenario
			overview, err := getFileContent(i, path.Join(theme.Name, edir, "introduction.txt"))
			if err != nil {
				errs = append(errs, fmt.Sprintf("%q: introduction.txt: %s", edir, err))
			}

			statement, err := getFileContent(i, path.Join(theme.Name, edir, "scenario.txt"))
			if err != nil {
				errs = append(errs, fmt.Sprintf("%q: scenario.txt: %s", edir, err))
				continue
			}

			// TODO: calculate gain
			var gain int64 = 42

			// TODO: handle video
			videoURI := ""

			if e, err := theme.GetExerciceByTitle(ename); err != nil {
				if ex, err := theme.AddExercice(ename, path.Join(theme.Name, edir), statement, overview, nil, gain, videoURI); err != nil {
					errs = append(errs, fmt.Sprintf("%q: error on exercice add: %s", edir, err))
					continue
				} else {
					dmap[eid] = ex
				}
			} else if e.Title != ename || e.Statement != statement || e.Overview != overview || e.Gain != gain || e.VideoURI != videoURI {
				e.Title = ename
				e.Statement = statement
				e.Overview = overview
				e.Gain = gain
				e.VideoURI = videoURI
				if _, err := e.Update(); err != nil {
					errs = append(errs, fmt.Sprintf("%q: error on exercice update: %s", edir, err))
					continue
				} else {
					dmap[eid] = e
				}
			} else {
				dmap[eid] = e
			}
		}

		// Remove old exercices
		if exercices, err := theme.GetExercices(); err == nil {
			for _, ex := range exercices {
				if _, ok := emap[ex.Title]; !ok {
					ex.Delete()
				}
			}
		}

		dmap_keys := []string{}
		for k, _ := range dmap {
			dmap_keys = append(dmap_keys, fmt.Sprintf("%d", k))
		}
		for _, e := range dmap {
			// Treat depends.txt
			if depends, err := getFileContent(i, path.Join(e.Path, "depends.txt")); err != nil {
				errs = append(errs, fmt.Sprintf("%q: depends.txt: %s", path.Base(e.Path), err))
			} else {
				for nline, dep := range strings.Split(depends, "\n") {
					if did, err := strconv.Atoi(dep); err != nil {
						errs = append(errs, fmt.Sprintf("%q: depends.txt:%d: %s", path.Base(e.Path), nline + 1, err))
						continue
					} else if exdep, exist := dmap[did]; !exist {
						errs = append(errs, fmt.Sprintf("%q: depends.txt:%d: Unable to find required dependancy %q (%d ; available: %s)", path.Base(e.Path), nline + 1, dep, did, strings.Join(dmap_keys, ",")))
						continue
					} else {
						e.Depend = &exdep.Id
						e.Update()
					}
				}
			}
		}
	}
	return errs
}

func ApiListRemoteExercices(theme fic.Theme, _ []byte) (interface{}, error) {
	return getExercices(GlobalImporter, theme)
}
